#! /bin/sh
#
#-------------------------------------------------------------------------------

# hgdemo V 1.0, Copyright (c) by E.G. (Manny) Noik, May 1993.
#
# This shell script demonstrates some of the capabilities of the
# C++ (hy)graph layout and manipulation filters.
#

#-------------------------------------------------------------------------------

# Variables
#

GS_LIB="$HPHOME/share/gs/fonts:$HPHOME/share/gs"; export GS_LIB
#psviewer="$HPHOME/bin/`mach`/ghostview -magstep -1"
psviewer="$HPHOME/bin/`mach`/ghostview"
psparms="-color"

tmp000=$HPHOME/tmp/tmp000
tmp001=$HPHOME/tmp/tmp001
tmp999=$HPHOME/tmp/tmp999

#-------------------------------------------------------------------------------

F_MainMenu() {

	while ( true ) do
		clear
		echo "hgdemo V 1.0,"
		echo "Copyright (c) May 1993 by E.G. (Manny) Noik,"
		echo
		echo "This shell script demonstrates some of the capabilities"
		echo "of the C++ (hy)graph layout and manipulation filters."
		echo
		echo
		echo "Select option: "
		echo
		echo "1 -- overview"
		echo "2 -- class hierarchy example"
		echo "3 -- medium-sized flat graph example"
		echo "4 -- small-sized flat graph example"
		echo
		echo "0 -- Quit"
		echo
		echo

		echo -n	"? "

		read answer

		case $answer in

			"0")
				echo "...quiting hgdemo."
				exit 0
				;;
			"1")
				F_Overview
				;;
			"2")
				F_ClassHierarchy
				;;
			"3")
				F_MediumSizedGraph
				;;
			"4")
				F_SmallSizedGraph
				;;
			*)
				;;
		esac
	clear
	done
}

#-------------------------------------------------------------------------------

F_Overview() {
	clear
	echo
	echo "Overview:"
	echo
	echo "the (hy)graph filter suite is implemented by approximately 12K"
	echo "lines of C++ source code and comprises three main programs:"
	echo
	echo "hglayout   -- draw (hy)graphs stored in GXF; the following"
	echo "              parametrized layout algorithms are currently"
	echo "              supported for both, 2D and 3D layout:"
	echo "              - random"
	echo "              - grid"
	echo "              - 1 dimensional"
	echo "              - stacking"
	echo "              - hierarchical"
	echo "              - 2D bin packing"
	echo "              - force-based"
	echo "              - overlap elimination"
	echo "              - circle"
	echo "              - nop (used in incremental composite layout)"
	echo "           -- these layout algorithms can be composed to"
	echo "              produce composite layouts"
	echo "           -- focal points can be specified to generate"
	echo "              non-geometric layout-independent fisheye views"
	echo

	echo -n "Hit <ENTER> to continue..."
	read answer

	clear
	echo "hgabstract -- perform several abstraction operations"
	echo "              on (hy)graphs stored in GXF:"
	echo "              - convert arcs to blobs"
	echo "              - convert blobs to arcs"
	echo "              - produce geometric fisheye views of (hy)graphs"
	echo "              - automatically hide node/blob contents based on"
	echo "                either size or nesting depth criteria"
	echo "              - declutter a (hy)graph through link-inheritance"
	echo "              - reset all layout information"
	echo
	echo "hgfileout  -- convert (hy)graph file format from GXF to:"
	echo "              - flat ascii file"
	echo "              - encapsulated PostScript"
	echo

	echo -n "Hit <ENTER> to continue..."
	read answer
}

#-------------------------------------------------------------------------------

F_ClassHierarchy() {
	clear
	echo
	echo "Class hierarchy:"
	echo
	echo "this example shows a typical object-oriented class hierarchy"
	echo "which contains 74 classes."
	echo

	cd $HPHOME/Data/imagedit

	echo
	echo "-> draw original flat graph using the force-based algorithm..."
	echo
	set -x
	hglayout -lspring < tree.gxf > $tmp000
	hgfileout $psparms < $tmp000 > $tmp999
	set +x
	$psviewer $tmp999

	echo
	echo "-> obtain a geometric fisheye view of the force-based layout..."
	echo
	set -x
	hgabstract -afisheye -id 50 -m < $tmp000 | hgfileout $psparms > $tmp999
	set +x
	$psviewer $tmp999

	echo
	echo "-> transform an arc relation into a blob relation and draw the"
	echo "   result using the stacking algorithm..."
	echo
	set -x
	hgabstract -aa2b sub 1 < tree.gxf | hglayout -lstack > $tmp000
	hgfileout $psparms < $tmp000 > $tmp999
	set +x
	$psviewer $tmp999

	echo
	echo "-> obtain a non-geometric fisheye view of the stack layout..."
	echo
	set -x
	hglayout -lstack -rank 50 8 1 e < $tmp000 | hgfileout $psparms > $tmp999
	set +x
	$psviewer $tmp999

	echo
	echo "-> transform the blobs back into arcs, but reverse arc direction;"
	echo "   draw the result using the hierarchical algorithm..."
	echo
	set -x
	hgabstract -ab2a sub 0 < $tmp000 > $tmp001
	hglayout -lhier -- x < $tmp001 | hgfileout $psparms -orientation 1 \
	> $tmp999
	set +x
	$psviewer $tmp999

	rm -f $tmp000 $tmp001 $tmp999
}

#-------------------------------------------------------------------------------

F_MediumSizedGraph() {
	clear
	echo
	echo "Medium-sized graph:"
	echo
	echo "this example shows several layouts of a medium-sized graph --"
	echo "the linkedto relation of a nuclear reactor system which consists"
	echo "of 204 nodes and 235 arcs"
	echo

	cd $HPHOME/Data/reactor

	echo
	echo "-> A hierarchical layout of the linkedto relation..."
	echo
	set -x
	hglayout -lhier < linkedto.gxf | hgfileout $psparms -orientation 2 \
	> $tmp999
	set +x
	$psviewer $tmp999

	echo
	echo "-> A force-based layout of the linkedto relation..."
	echo
	set -x
	hglayout -lspring < linkedto.gxf | hgfileout $psparms > $tmp999
	set +x
	$psviewer $tmp999

	rm -f $tmp000 $tmp001 $tmp999
}

#-------------------------------------------------------------------------------

F_SmallSizedGraph() {
	clear
	echo
	echo "Small-sized graph:"
	echo
	echo "this example shows several layouts of a small-sized graph --"
	echo "a flights database which consists of 36 nodes and 103 arcs"
	echo

	cd $HPHOME/Data/flights

	echo
	echo "-> a grid layout of the flights database..."
	echo
	set -x
	hglayout -- minsep 20 20 20 < flights.arcs.gxf | hgfileout $psparms \
	> $tmp999
	set +x
	$psviewer $tmp999

	echo
	echo "-> a circle layout of the flights database..."
	echo
	set -x
	hglayout -lcircle -- reorder < flights.arcs.gxf | hgfileout $psparms \
	> $tmp999
	set +x
	$psviewer $tmp999

	echo
	echo "-> a hierarchical layout of the flights database..."
	echo
	set -x
	hglayout -lhier < flights.arcs.gxf | hgfileout $psparms -orientation 2 \
	> $tmp999
	set +x
	$psviewer $tmp999

	echo
	echo "-> a force-based layout of the flights database..."
	echo
	set -x
	hglayout -lrandom < flights.arcs.gxf | hglayout -lspring -- e 130 | \
	hgfileout $psparms > $tmp999
	set +x
	$psviewer $tmp999

	echo
	echo "-> generate a nested flights database and draw the result"
	echo "   with the grid layout algorithm..."
	echo
	set -x
	hgabstract -aa2b city 0 < flights.arcs.gxf | \
	hgabstract -aa2b country 0 | \
	hglayout -lgrid -- minsep 20 20 20 > $tmp000
	hgfileout $psparms < $tmp000 > $tmp999
	set +x
	$psviewer $tmp999

	echo
	echo "-> a stack layout of the nested flights database..."
	echo
	set -x
	hglayout -lstack < $tmp000 | hgfileout $psparms > $tmp999
	set +x
	$psviewer $tmp999

	echo
	echo "-> a 2D bin packing layout of the nested flights database..."
	echo
	set -x
	hglayout -lpack minsep 15 15 15 < $tmp000 | hgfileout $psparms > $tmp999
	set +x
	$psviewer $tmp999

	echo
	echo "-> a composite layout of the nested flights database..."
	echo
	echo "     South America          :  force-based"
	echo "     Europe                 :  random"
	echo "     Far East               :  horizontal"
	echo "     United States          :  ellipse"
	echo "     Canada                 :  hierarchical"
	echo "     North America and world:  vertical"
        echo
	set -x
	hglayout -lstack < flights.comp.gxf | hgfileout $psparms > $tmp999
	set +x
	$psviewer $tmp999

	echo
	echo "-> a stack layout of the nested flights database in which"
	echo "   Toronto and Hong Kong have been emphasized..."
	echo
	set -x
	hglayout -lstack -rank 5 9 1 e -rank 15 9 1 e < $tmp000 | \
	hgfileout $psparms > $tmp999
	set +x
	$psviewer $tmp999

	echo
	echo "-> a stack layout of the nested flights database in which the"
	echo "   contents of vertices nested at level 1 have been hidden..."
	echo
	set -x
	hglayout -lstack < $tmp000 | hgabstract -ahide -l 1 |
	hgfileout $psparms > $tmp999
	set +x
	$psviewer $tmp999

	echo
	echo "-> a hierarchical layout of the nested flights database in which"
	echo "   the number of edges has been reduced by applying link"
	echo "   inheritance..."
	echo
	set -x
	hgabstract -ali < $tmp000 | hglayout -lhier | hgfileout $psparms \
	-orientation 2 > $tmp999
	set +x
	$psviewer $tmp999

	rm -f $tmp000 $tmp001 $tmp999
}

#-------------------------------------------------------------------------------

# Top-level program
#

F_MainMenu

exit 0

#-------------------------------------------------------------------------------
